var T_FILE = "zxnz_core_fs_File";
var T_FOLDER = "zxnz_core_fs_Folder";
var node_fs = require('fs');
var node_path = require('path');
var mimes = require('./mimes');
var archiver = require('archiver');
module.exports = zn.Controller('download', {
    methods: {
        file: {
            method: 'GET/POST',
            route: '/file/:tempName',
            argv: {
                tempName: null
            },
            value: function (request, response, application, context, router){
                var _tempName = request.getValue('tempName'),
                    _downloadName = request.getValue('downloadName');
                zxnz.store.beginPoolTransaction()
                    .query("select configs by namespace", function (){
                        return zxnz.sql.select({
                            table: T_FILE,
                            where: {
                                zxnz_core_fs_file_Temp_Name: _tempName
                            }
                        });
                    })
                    .commit()
                    .then(function (data){
                        if(!data.length){
                            return response.error('文件不存在！'), false;
                        }else{
                            var _file = data[0];
                            if(!node_fs.existsSync(_file.zxnz_core_fs_file_Saved_Path)){
                                return response.error('文件已经被删除, 请联系管理员'), false;
                            }
                            var _serverResponse = response._serverResponse,
                                _fileStat = node_fs.statSync(_file.zxnz_core_fs_file_Saved_Path);
                            if(!_fileStat.isFile()){
                                return response.error('该请求文件路径出错, 请联系管理员'), false;
                            }
                            _serverResponse.writeHead(200, {
                                "Content-Type": _file.zxnz_core_fs_file_Type,
                                "Content-Length": _fileStat.size,
                                "Content-Disposition": "attachment; filename=" + encodeURI(_downloadName || _file.zxnz_core_fs_file_Name)
                            });
                            node_fs.createReadStream(_file.zxnz_core_fs_file_Saved_Path).pipe(_serverResponse);
                        }
                    }, function (err){
                        response.error(err);
                    });
            }
        },
        downloadByPath: {
            method: 'GET/POST',
            route: '/bypath',
            argv: {
                path: null
            },
            value: function (request, response, application, context, router){
                var _config = context.getFileUploadConfig();
                var _path = node_path.join(_config.savedDir, request.getValue('path'));
                if(!node_fs.existsSync(_path)){
                    return response.error('文件不存在');
                }
                var _serverResponse = response._serverResponse,
                    _fileName = request.getValue('name') || node_path.basename(_path),
                    _fileExt = node_path.extname(_path),
                    _fileStat = node_fs.statSync(_path);
                if(!_fileStat.isFile()){
                    return response.error('该请求文件路径出错, 请联系管理员'), false;
                }
                _serverResponse.writeHead(200, {
                    "Content-Type": mimes[_fileExt] + ';chartset=utf8',
                    "Content-Length": _fileStat.size,
                    "Content-Disposition": "attachment; filename=" + encodeURI(_fileName)
                });
                node_fs.createReadStream(_path).pipe(_serverResponse);
            }
        },
        downloadZipByFolder: {
            method: 'GET/POST',
            argv: {
                folder: null,
                name: null
            },
            value: function (request, response, application, context, router){
                var _config = context.getFileUploadConfig();
                var _path = node_path.join(_config.savedDir, request.getValue('folder'));
                console.log('Path: ', _path);
                if(!node_fs.existsSync(_path)){
                    return response.error('文件不存在');
                }
                var _fileStat = node_fs.statSync(_path);
                if(!_fileStat.isDirectory()){
                    return response.error('该请求路径不是目录, 请求错误'), false;
                }
                
                var _serverResponse = response._serverResponse;
                _serverResponse.writeHead(200, {
                    'Content-Type': 'application/x-zip',
                    'Pragma': 'public',
                    'Expires': '0',
                    'Cache-Control': 'private, must-revalidate, post-check=0, pre-check=0',
                    'Content-disposition': 'attachment; filename="' + encodeURI(request.getValue('name')) + '.zip"',
                    'Transfer-Encoding': 'chunked',
                    'Content-Transfer-Encoding': 'binary'
                });

                var archive = archiver('zip', {
                    zlib: { level: 9 } // Sets the compression level.
                });
                archive.on('error', function(err) {
                    zn.error(err);
                });
                archive.on('finish', function(err) {
                    if(err){
                        zn.error(err);
                    }else{
                        _serverResponse.end();
                    }
                });
                archive.store = false;
                archive.pipe(_serverResponse);
                archive.directory(_path, false);
                /*
                var _dir = node_fs.readdirSync(_path);
                _dir.map(function (file){
                    archive.append(node_fs.createReadStream(_path + file), {
                        name: file
                    });
                });*/
                archive.finalize();
            }
        },
        downloadZipByFiles: {
            method: 'GET/POST',
            argv: {
                files: '',
                filename: '文件',
                downloadName: '打包下载'
            },
            value: function (request, response, application, context, router){
                var _config = context.getFileUploadConfig(),
                    _filename = decodeURI(request.getValue('filename')),
                    _files = request.getValue('files').split(',');
                zn.debug(_filename, _files, request.getValue('downloadName'));
                //response.success('ok');
                //return;

                var _serverResponse = response._serverResponse;
                _serverResponse.writeHead(200, {
                    'Content-Type': 'application/x-zip',
                    'Pragma': 'public',
                    'Expires': '0',
                    'Cache-Control': 'private, must-revalidate, post-check=0, pre-check=0',
                    'Content-disposition': 'attachment; filename="' + encodeURI(request.getValue('downloadName')) + '.zip"',
                    'Transfer-Encoding': 'chunked',
                    'Content-Transfer-Encoding': 'binary'
                });

                var archive = archiver('zip', {
                    zlib: { level: 9 } // Sets the compression level.
                });
                // good practice to catch warnings (ie stat failures and other non-blocking errors)
                archive.on('warning', function(err) {
                    zn.debug(err);
                    if (err.code === 'ENOENT') {
                        // log warning
                    } else {
                        // throw error
                        throw err;
                    }
                });
                // good practice to catch this error explicitly
                archive.on('error', function(err) {
                    zn.error(err);
                    throw err;
                });
                archive.on('finish', function(err) {
                    if(err){
                        zn.error(err);
                    }else{
                        _serverResponse.end();
                    }
                });
                archive.store = false;
                archive.pipe(_serverResponse);
                //archive.pipe(node_fs.createWriteStream(__dirname + '/example.zip'));
                _files.forEach(function (file, index){
                    if(file){
                        var _path = context.jsonWriter.getKey(file);
                        if(node_fs.existsSync(_path)){
                            zn.debug('Path: ', _path, _filename + '_' + index + node_path.extname(_path));
                            archive.append(node_fs.createReadStream(_path), {
                                name: _filename + '_' + index + node_path.extname(_path)
                            });
                        }
                    }
                });
                archive.finalize();
            }
        }
    }
});